#define QT_FEATURE_alsa 1
#define QT_FEATURE_directshow -1
#define QT_FEATURE_directshow_player -1
#define QT_FEATURE_evr -1
#define QT_FEATURE_gpu_vivante -1
#define QT_FEATURE_gstreamer_1_0 1
#define QT_FEATURE_gstreamer 1
#define QT_FEATURE_gstreamer_0_10 -1
#define QT_FEATURE_gstreamer_app 1
#define QT_FEATURE_gstreamer_encodingprofiles 1
#define QT_FEATURE_gstreamer_photography -1
#define QT_FEATURE_linux_v4l 1
#define QT_FEATURE_openal -1
#define QT_FEATURE_pulseaudio -1
#define QT_FEATURE_resourcepolicy -1
#define QT_FEATURE_wasapi -1
#define QT_FEATURE_wmf -1
#define QT_FEATURE_wmf_player -1
#define QT_FEATURE_wmsdk -1
#define QT_FEATURE_wshellitem -1
