/* This file was generated by syncqt. */
#ifndef QT_QTWEBENGINECORE_VERSION_H
#define QT_QTWEBENGINECORE_VERSION_H

#define QTWEBENGINECORE_VERSION_STR "5.9.6"

#define QTWEBENGINECORE_VERSION 0x050906

#endif // QT_QTWEBENGINECORE_VERSION_H
